/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * pro-flash.c
 * Copyright (C) 2005-2006 Simon Newton
 */

#include "pro-common.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>
#include <time.h>


#define DEFAULT_DEVICE "/dev/ttyUSB0" 
#define DEFAULT_IMAGE "main.bin"

typedef struct {
	int verbose;
	int help;
	char *device;
	char *image;
} options;

// number of seconds to wait for a firmware response before aborting
const int TIMEOUT = 2;


int got_rep = 0 ;


// our command line options
options opts ;


/*
 * Called on a flash page reply
 */
int handle_flash_reply(pro_device dev, pmu *msg, int len) {
	
	if(memcmp( msg->pmu_flsrep.res , "TRUE", sizeof(msg->pmu_flsrep.res) ) == 0 ) {
		got_rep = 1 ;
	} else {
		got_rep = 2 ;
	}
	return 0;

}

/*
 * Send the flash pages
 */
int reprogram_flash(pro_device dev, int fd) {
	uint8_t page[FLASH_PAGE] ;
	int sz ;
	struct timeval start, now;
	
	while( (sz = read(fd, page, FLASH_PAGE)) > 0 ) {
		got_rep = 0 ;

		gettimeofday(&start, NULL);
		start.tv_sec += TIMEOUT;
		
		while (got_rep != 1) {
			printf(".") ;
			fflush(NULL) ;

			gettimeofday(&now, NULL);
			if (timercmp(&now, &start, >= )) {
				printf(" Device not responding, this looks like an Open Widget\n");
				return -1;
			}
			pro_send_flsreq(dev, page, sz) ;

			while (! got_rep) {
				if ( pro_recv(dev,1) == 1) {
					break ;
				}
			}
		}	
	}
	printf("\n") ;
	return 0 ;
}


/*
 * Init options
 */
void init_options(options *opts) {
	opts->verbose =  0 ;
	opts->help =  0 ;
	opts->device = DEFAULT_DEVICE ;
	opts->image = DEFAULT_IMAGE ;
	
}

/*
 * parse our cmd line options
 *
 */
int parse_options(int argc, char *argv[], options *opts) {
	static struct option long_options[] = {
			{"verbose", 	no_argument, 		0, 'v'},
			{"help", 		no_argument, 		0, 'h'},
			{0, 0, 0, 0}
		};

	int c ;
	int option_index = 0;
	
	while (1) {
     
		c = getopt_long(argc, argv, "vh", long_options, &option_index);
		
		if (c == -1)
			break;

		switch (c) {
			case 0:
				break;
			case 'v':
				opts->verbose = 1 ;
				break ;
			case 'h':
				opts->help = 1 ;
				break;
			case '?':
				break;
     
			default:
				;
		}
	}

	if (optind < argc) {
		if( optind + 1 == argc) {
			opts->image = argv[optind++] ;
		} else {
			opts->device = argv[optind++] ;
			opts->image = argv[optind++] ;
		}
	}

	return 0;
}


/*
 * Display the help message
 */
void display_help_and_exit(char *argv[]) {

	printf(
"Usage: %s [device] [firmware]\n"
"\n"
"Flash the firmware on an Enttec USB Pro device.\n"
"\n"
"  -h, --help        Display this help message and exit.\n"
"  -v, --verbose     Be verbose.\n"
"\n"
"Defaults:\n"
" device        /dev/ttyUSB0\n"
" firmware      main.bin\n",
		argv[0]) ;
	
	exit(0);
}



/*
 * Flashes the device
 *
 */
int main(int argc , char *argv[] ) {
	int  fd ;
	pro_device *dev ;

	init_options(&opts);
	parse_options(argc, argv, &opts) ;

	if(opts.help)
		display_help_and_exit(argv) ;

	fd = open(opts.image , O_RDONLY) ;

	if( fd < 0) {
		printf("Couldn't open firmware file %s : %s\n", opts.image, strerror(errno) ) ;
		exit(1) ;
	}

	dev = pro_connect(opts.device, opts.verbose) ;

	if( dev == NULL) {
		return 1 ;
	}

	pro_set_handler(dev, PRO_FLSREP_H , handle_flash_reply) ;

	pro_send_reprogram(dev) ;

	reprogram_flash(dev, fd) ;
	
		
	pro_disconnect(dev) ;
	return 0 ;

}
